@@ -151,9 +151,8 @@ inline void removeIndexFromRow(const QModelIndex &index, const QVector<int> &rol
         if (roles.isEmpty()) {
             entry.data.clear();
         } else {
-            Q_FOREACH (int role, roles) {
+            for (int role : roles)
                 entry.data.remove(role);
-            }
         }
     }
 }
@@ -384,6 +383,16 @@ void QAbstractItemModelReplicaImplementation::handleModelResetDone(QRemoteObject
     m_rootItem.columnCount = size.width();
     m_headerData[0].resize(size.width());
     m_headerData[1].resize(size.height());
+    {
+        QVector<CacheEntry> &headerEntries = m_headerData[0];
+        for (int i = 0; i < size.width(); ++i )
+            headerEntries[i].data.clear();
+    }
+    {
+        QVector<CacheEntry> &headerEntries = m_headerData[1];
+        for (int i = 0; i < size.height(); ++i )
+            headerEntries[i].data.clear();
+    }
     if (m_initialAction == QtRemoteObjects::PrefetchData) {
         auto entries = watcher->returnValue().value<MetaAndDataEntries>();
         for (int i = 0; i < entries.data.size(); ++i)
@@ -403,7 +412,7 @@ void QAbstractItemModelReplicaImplementation::handleSizeDone(QRemoteObjectPendin
 
     if (size.width() != parentItem->columnCount) {
         const int columnCount = std::max(0, parentItem->columnCount);
-        Q_ASSERT_X(size.width() >= parentItem->columnCount, __FUNCTION__, qPrintable(QStringLiteral("The column count should only shrink in columnsRemoved!!")));
+        Q_ASSERT_X(size.width() >= parentItem->columnCount, __FUNCTION__, "The column count should only shrink in columnsRemoved!!");
         parentItem->columnCount = size.width();
         if (size.width() > columnCount) {
             Q_ASSERT(size.width() > 0);
@@ -414,7 +423,7 @@ void QAbstractItemModelReplicaImplementation::handleSizeDone(QRemoteObjectPendin
         }
     }
 
-    Q_ASSERT_X(size.height() >= parentItem->rowCount, __FUNCTION__, qPrintable(QStringLiteral("The new size and the current size should match!!")));
+    Q_ASSERT_X(size.height() >= parentItem->rowCount, __FUNCTION__, "The new size and the current size should match!!");
     if (!parentItem->rowCount) {
         if (size.height() > 0) {
             q->beginInsertRows(parent, 0, size.height() - 1);
@@ -571,7 +580,7 @@ void QAbstractItemModelReplicaImplementation::fetchPendingData()
 
     std::vector<RequestedData> finalRequests;
     RequestedData curData;
-    Q_FOREACH (const RequestedData &data, m_requestedData) {
+    for (const RequestedData &data : qExchange(m_requestedData, {})) {
         qCDebug(QT_REMOTEOBJECT_MODELS) << Q_FUNC_INFO << "REQUESTED start=" << data.start << "end=" << data.end << "roles=" << data.roles;
 
         Q_ASSERT(!data.start.isEmpty());
@@ -601,7 +610,7 @@ void QAbstractItemModelReplicaImplementation::fetchPendingData()
             const ModelIndex resEnd(std::max(curIndEnd.row, dataIndEnd.row), std::max(curIndEnd.column, dataIndEnd.column));
             QVector<int> roles = curData.roles;
             if (!curData.roles.isEmpty()) {
-                Q_FOREACH (int role, data.roles) {
+                for (int role : data.roles) {
                     if (!curData.roles.contains(role))
                         roles.append(role);
                 }
@@ -645,7 +654,6 @@ void QAbstractItemModelReplicaImplementation::fetchPendingData()
         m_pendingRequests.push_back(watcher);
         connect(watcher, &RowWatcher::finished, this, &QAbstractItemModelReplicaImplementation::requestedData);
     }
-    m_requestedData.clear();
 }
 
 void QAbstractItemModelReplicaImplementation::onModelReset()
@@ -673,7 +681,7 @@ void QAbstractItemModelReplicaImplementation::fetchPendingHeaderData()
     QVector<int> roles;
     QVector<int> sections;
     QVector<Qt::Orientation> orientations;
-    Q_FOREACH (const RequestedHeaderData &data, m_requestedHeaderData) {
+    for (const RequestedHeaderData &data : qAsConst(m_requestedHeaderData)) {
         roles.push_back(data.role);
         sections.push_back(data.section);
         orientations.push_back(data.orientation);
